/*
 * Line6 Linux USB driver - 0.6.5
 *
 * Copyright (C) 2004-2006 Markus Grabner (grabner@icg.tu-graz.ac.at)
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License as
 *	published by the Free Software Foundation, version 2.
 *
 */

#ifndef DRIVER_H
#define DRIVER_H


#include <linux/spinlock.h>
#include <linux/usb.h>
#include <linux/wait.h>

#include <sound/driver.h>
#include <sound/core.h>

#include "config.h"
#include "midi.h"


#define DRIVER_NAME "line6usb"

#define LINE6_TIMEOUT 1
#define LINE6_MAX_DEVICES 8
#define LINE6_BUFSIZE_LISTEN 32
#define LINE6_MESSAGE_MAXLEN 256

/*
	Line6 MIDI control commands
*/
#define LINE6_PARAM_CHANGE   0xb0
#define LINE6_PROGRAM_CHANGE 0xc0
#define LINE6_SYSEX_BEGIN    0xf0
#define LINE6_SYSEX_END      0xf7
#define LINE6_RESET          0xff

#define LINE6_OFFSET_ECHO 0
#define LINE6_OFFSET_ORIG 2
#define LINE6_OFFSET_UNKNOWN 5  /* don't know yet what this is good for */


extern const unsigned char line6_sysex_header[3];

static const int SYSEX_DATA_OFS = sizeof(line6_sysex_header) + 3;
static const int SYSEX_EXTRA_SIZE = sizeof(line6_sysex_header) + 4;


/**
	 Common data shared by all Line6 devices.
*/
struct usb_line6 {
	/**
		 USB device.
	*/
	struct usb_device *usbdev;

	/**
		 Name of this device.
	*/
	const char *devname;

	/**
		 Bit code of this device.
	*/
	int devbit;

	/**
		 Device to send messages to (via dev_*).
	*/
	struct device *msgdev;

	/**
		 Line6 sound card data structure.
		 Each device has at least a MIDI connection.
	*/
	snd_card_t *card;

	/**
		 Line6 MIDI device data structure.
	*/
	struct snd_line6_midi *midi;

	/**
		 USB endpoint for listening to control commands.
	*/
	int ep_control_read;

	/**
		 USB endpoint for writing control commands.
	*/
	int ep_control_write;

	/**
		 URB for listening to PODxt Pro control endpoint.
	*/
	struct urb *urb_listen;

	/**
		 Buffer for listening to PODxt Pro control endpoint.
	*/
	unsigned char *buffer_listen;

	/**
		 Buffer for composition of messages.
	*/
	unsigned char *buffer_message;

	/**
		 Position in message composition buffer.
	*/
	int message_pos;
	
	/**
		 Length of message to be composed.
		 0:  no current message, determine length with next received byte
		 >0: total length of current message (if known in advance)
		 -1: length unknown (wait for sysex end)
	*/
	int message_length;
};


extern void *line6_devices[LINE6_MAX_DEVICES];

extern char *line6_alloc_sysex_buffer(struct usb_line6 *line6, int code1, int code2, int size);
extern ssize_t line6_nop_read(struct device *dev, DEVICE_ATTRIBUTE char *buf);
extern ssize_t line6_nop_write(struct device *dev, DEVICE_ATTRIBUTE const char *buf, size_t count);
extern int line6_send_program(struct usb_line6 *line6, int value);
extern int line6_send_raw_message(struct usb_line6 *line6, const char *buffer, int size);
extern int line6_send_raw_message_async(struct usb_line6 *line6, const char *buffer, int size);
extern int line6_send_sysex_message(struct usb_line6 *line6, const char *buffer, int size);
extern ssize_t line6_set_raw(struct device *dev, DEVICE_ATTRIBUTE const char *buf, size_t count);
extern int line6_transmit_parameter(struct usb_line6 *line6, int param, int value);
extern void line6_write_hexdump(struct usb_line6 *line6, char dir, const unsigned char *buffer, int size);


/* use usb_maxpacket (usb.h) or similar instead!!! */
#define LINE6_RAW_CHUNK_SIZE 32


#endif
