/*
 * Line6 Linux USB driver - 0.6.5
 *
 * Copyright (C) 2004-2006 Markus Grabner (grabner@icg.tu-graz.ac.at)
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License as
 *	published by the Free Software Foundation, version 2.
 *
 */

#include "config.h"

#include "audio.h"
#include "capture.h"
#include "control.h"
#include "driver.h"
#include "playback.h"
#include "pod.h"


#define POD_SYSEX_CODE 3

enum {
	POD_SYSEX_SAVE      = 0x24,
	POD_SYSEX_SYSTEM    = 0x56,
	POD_SYSEX_SYSTEMREQ = 0x57,
	/* POD_SYSEX_UPDATE    = 0x6c, */  /* software update! */
	POD_SYSEX_STORE     = 0x71,
	POD_SYSEX_FINISH    = 0x72,
	POD_SYSEX_DUMPMEM   = 0x73,
	POD_SYSEX_DUMP      = 0x74,
	POD_SYSEX_DUMPREQ   = 0x75
	/* POD_SYSEX_DUMPMEM2  = 0x76 */   /* dumps entire internal memory of PODxt Pro */
};

enum {
	POD_monitor_level  = 0x04,
	POD_routing        = 0x05,
	POD_tuner_mute     = 0x13,
	POD_tuner_freq     = 0x15,
	POD_tuner_note     = 0x16,
	POD_tuner_pitch    = 0x17,
	POD_system_invalid = 0x7fff
};

enum {
	POD_DUMP_MEMORY = 2
};

enum {
  POD_BUSY_READ,
  POD_BUSY_WRITE,
  POD_CHANNEL_DIRTY,
  POD_SAVE_PRESSED,
  POD_BUSY_MIDISEND
};


static const char pod_request_version[] = { 0xf0, 0x7e, 0x7f, 0x06, 0x01, 0xf7 };
static const char pod_request_channel[] = { 0xf0, 0x00, 0x01, 0x0c, 0x03, 0x75, 0xf7 };
static const char pod_version_header [] = { 0xf2, 0x7e, 0x7f, 0x06, 0x02 };


/*
	Mark all parameters as dirty and notify waiting processes.
*/
static void pod_mark_batch_all_dirty(struct usb_line6_pod *pod)
{
	int i;

	for(i = POD_CONTROL_SIZE; i--;)
		set_bit(i, pod->param_dirty);

	wake_up_interruptible(&pod->batch_wait);
}

/*
	Send an asynchronous request for the POD firmware version and device ID.
*/
static int pod_version_request_async(struct usb_line6_pod *pod)
{
	return line6_send_raw_message_async(&pod->line6, pod->buffer_versionreq, sizeof(pod_request_version));
}

static void pod_startup_timeout(unsigned long arg)
{
	static int count = 0;
	struct usb_line6_pod *pod = (struct usb_line6_pod *)arg;

	if(pod->dumpreq.ok && pod->versionreq_ok)
		return;

	if(count++ & 1)
		line6_dump_request_async(&pod->dumpreq, &pod->line6, 0);
	else
		pod_version_request_async(pod);

	line6_startup_delayed(&pod->dumpreq, 1, pod_startup_timeout, pod);
}

static char *pod_alloc_sysex_buffer(struct usb_line6_pod *pod, int code, int size)
{
	return line6_alloc_sysex_buffer(&pod->line6, POD_SYSEX_CODE, code, size);
}

/*
	Send channel dump data to the PODxt Pro.
*/
static void pod_dump(struct usb_line6_pod *pod, const unsigned char *data)
{
	int size = 1 + sizeof(pod->prog_data);
	char *sysex = pod_alloc_sysex_buffer(pod, POD_SYSEX_DUMP, size);
	if(!sysex) return;
	sysex[SYSEX_DATA_OFS] = 5;  /* Don't know what this is good for, but PODxt Pro transmits it, so we also do... */
	memcpy(sysex + SYSEX_DATA_OFS + 1, data, sizeof(pod->prog_data));
	line6_send_sysex_message(&pod->line6, sysex, size);
	memcpy(&pod->prog_data, data, sizeof(pod->prog_data));
	pod_mark_batch_all_dirty(pod);
	kfree(sysex);
}

/*
	Store parameter value in driver memory and mark it as dirty.
*/
static void pod_store_parameter(struct usb_line6_pod *pod, int param, int value)
{
	pod->prog_data.control[param] = value;
	set_bit(param, pod->param_dirty);
	pod->dirty = 1;
	wake_up_interruptible(&pod->batch_wait);
}

/*
	Handle SAVE button
*/
static void pod_save_button_pressed(struct usb_line6_pod *pod, int type, int index)
{
	pod->dirty = 0;
	set_bit(POD_SAVE_PRESSED, &pod->atomic_flags);
	wake_up_interruptible(&pod->batch_wait);
}

/*
	Process a completely received message.
*/
void pod_process_message(struct usb_line6_pod *pod)
{
	const unsigned char *buf = pod->line6.buffer_message;

	switch(buf[0]) {
	case LINE6_PARAM_CHANGE | LINE6_OFFSET_ORIG:
		pod_store_parameter(pod, buf[1], buf[2]);
		/* intentionally no break here! */

	case LINE6_PARAM_CHANGE | LINE6_OFFSET_ECHO:
		if((buf[1] == POD_amp_model_setup) || (buf[1] == POD_effect_setup))  /* these also affect other settings */
			line6_dump_request_async(&pod->dumpreq, &pod->line6, 0);

		break;

	case LINE6_PROGRAM_CHANGE | LINE6_OFFSET_ORIG:
	case LINE6_PROGRAM_CHANGE | LINE6_OFFSET_ECHO:
		pod->channel_num = buf[1];
		pod->dirty = 0;
		set_bit(POD_CHANNEL_DIRTY, &pod->atomic_flags);
		line6_dump_request_async(&pod->dumpreq, &pod->line6, 0);
		break;

	case LINE6_SYSEX_BEGIN | LINE6_OFFSET_ORIG:
	case LINE6_SYSEX_BEGIN | LINE6_OFFSET_UNKNOWN:
		if(memcmp(buf + 1, line6_sysex_header, sizeof(line6_sysex_header)) == 0) {
			switch(buf[5]) {
			case POD_SYSEX_DUMP:
				if(pod->line6.message_pos == sizeof(pod->prog_data) + 8) {
					switch(pod->dumpreq.in_progress) {
					case LINE6_DUMP_CURRENT:
						memcpy(&pod->prog_data, buf + 7, sizeof(pod->prog_data));
						pod_mark_batch_all_dirty(pod);
						pod->dumpreq.ok = 1;
						break;

					case POD_DUMP_MEMORY:
						memcpy(&pod->prog_data_buf, buf + 7, sizeof(pod->prog_data_buf));
						break;

					default:
						DEBUG_MESSAGES(dev_err(pod->line6.msgdev, "unknown dump code %02X\n", pod->dumpreq.in_progress));
					}

					line6_dump_finished(&pod->dumpreq);
				}
				else
					DEBUG_MESSAGES(dev_err(pod->line6.msgdev, "wrong size of channel dump message (%d instead of %d)\n",
																 pod->line6.message_pos, (int)sizeof(pod->prog_data) + 8));
							
				break;

			case POD_SYSEX_SYSTEM: {
				short value = ((int)buf[7] << 12) | ((int)buf[8] << 8) | ((int)buf[9] << 4) | (int)buf[10];

#define PROCESS_SYSTEM_PARAM(x) \
					case POD_ ## x: \
						pod->x.value = value; \
						wake_up_interruptible(&pod->x.wait); \
						break;

				switch(buf[6]) {
					PROCESS_SYSTEM_PARAM(monitor_level);
					PROCESS_SYSTEM_PARAM(routing);
					PROCESS_SYSTEM_PARAM(tuner_mute);
					PROCESS_SYSTEM_PARAM(tuner_freq);
					PROCESS_SYSTEM_PARAM(tuner_note);
					PROCESS_SYSTEM_PARAM(tuner_pitch);

#undef PROCESS_SYSTEM_PARAM

				default:
					DEBUG_MESSAGES(dev_err(pod->line6.msgdev, "unknown tuner/system response %02X\n", buf[6]));
				}

				break;
			}
				
			case POD_SYSEX_FINISH:
				/* do we need to respond to this? */
				break;

			case POD_SYSEX_SAVE:
				pod_save_button_pressed(pod, buf[6], buf[7]);
				break;

			case POD_SYSEX_STORE:
				DEBUG_MESSAGES(dev_err(pod->line6.msgdev, "message %02X not yet implemented\n", buf[5]));
				break;

			default:
				DEBUG_MESSAGES(dev_err(pod->line6.msgdev, "unknown sysex message %02X\n", buf[5]));
			}
		}
		else if(memcmp(buf, pod_version_header, sizeof(pod_version_header)) == 0) {
			if(pod->versionreq_ok == 0) {
				pod->firmware_version = buf[13] * 100 + buf[14] * 10 + buf[15];
				pod->device_id = ((int)buf[8] << 16) | ((int)buf[9] << 8) | (int)buf[10];
				// now we know the firmware version, so let's create the special files:
				pod_create_files(pod->firmware_version, pod->line6.devbit, pod->line6.msgdev);
				pod->versionreq_ok = 1;
			}
			else
				DEBUG_MESSAGES(dev_err(pod->line6.msgdev, "multiple firmware version message\n"));
		}
		else
			DEBUG_MESSAGES(dev_err(pod->line6.msgdev, "unknown sysex header\n"));
			
		break;

	default:
		DEBUG_MESSAGES(dev_err(pod->line6.msgdev, "unknown message %02X\n", buf[0]));
	}
}

/*
	Detect some cases that require a channel dump.
	Important notes:
	*) The actual dump request can not be sent here since we are not allowed to
	wait for the completion of the first message in this context, and sending
	the dump request before completion of the previous message leaves the POD
	in an undefined state. The dump request will be sent when the echoed
	commands are received.
	*) This method fails if a param change message is "chopped" after the first
	byte.
*/
void pod_midi_postprocess(struct usb_line6_pod *pod, unsigned char *data, int length)
{
	int i;

	if(!pod->midi_postprocess)
		return;

	for(i = 0; i < length; ++i) {
		if(data[i] == LINE6_PROGRAM_CHANGE) {
			line6_invalidate_current(&pod->dumpreq);
			break;
		}
		else if((data[i] == LINE6_PARAM_CHANGE) && (i < length - 1))
			if((data[i + 1] == POD_amp_model_setup) || (data[i + 1] == POD_effect_setup)) {
				line6_invalidate_current(&pod->dumpreq);
				break;
			}
	}
}

/*
	Send channel number (i.e., switch to a different sound).
*/
void pod_send_channel(struct usb_line6_pod *pod, int value)
{
	line6_invalidate_current(&pod->dumpreq);
	
	if(line6_send_program(&pod->line6, value) == 0)
		pod->channel_num = value;
	else
		line6_dump_finished(&pod->dumpreq);
}

/*
	Transmit PODxt Pro control parameter.
*/
void pod_transmit_parameter(struct usb_line6_pod *pod, int param, int value)
{
	if(line6_transmit_parameter(&pod->line6, param, value) == 0)
		pod_store_parameter(pod, param, value);

	if((param == POD_amp_model_setup) || (param == POD_effect_setup))  /* these also affect other settings */
		line6_invalidate_current(&pod->dumpreq);
}

/*
	Read POD serial number.
*/
static void pod_read_serial_number(struct usb_line6_pod *pod)
{
	struct usb_device *usbdev = pod->line6.usbdev;
	int ret;

	/* for some reason, this message needs to be sent before querying the serial number: */
	ret = usb_control_msg(usbdev, usb_sndctrlpipe(usbdev, 0), 0x67,
												USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_DIR_OUT,
												0x0421, 0x80d0, 0, 0, LINE6_TIMEOUT * HZ);

	if(ret < 0) {
		dev_err(pod->line6.msgdev, "reading of serial number failed (step 1, error %d)\n", ret);
		return;
	}

	/* just waiting for USB completion is not sufficient :-( */
	msleep(10);

	/* now query the serial number: */
	ret = usb_control_msg(usbdev, usb_rcvctrlpipe(usbdev, 0), 0x67,
												USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_DIR_IN,
												0x0013, 0x0000, &pod->serial_number, 4, LINE6_TIMEOUT * HZ);

	if(ret < 0) {
		dev_err(pod->line6.msgdev, "reading of serial number failed (step 2, error %d)\n", ret);
		return;
	}
}

#if USE_CHARACTER_DEVICE

/*
	Determine message length.
*/
static ssize_t pod_message_length(unsigned char cmd)
{
	switch(cmd) {
	case LINE6_PARAM_CHANGE: return 3;
	case LINE6_PROGRAM_CHANGE: return 2;
	default: return -1;
	}
}

/*
	Copy data to user space for read call.
*/
static void pod_read_copy(struct usb_line6_pod *pod, unsigned char **bufp, size_t *count)
{
	size_t mlen = pod->chrdev_msglen;
	size_t len = min(*count, mlen - pod->chrdev_readpos);
	int bytes_failed = copy_to_user(*bufp, pod->chrdev_readbuf + pod->chrdev_readpos, len);

	if(bytes_failed > 0) {
		DEBUG_MESSAGES(dev_err(pod->line6.msgdev, "couldn't copy data\n"));
		len -= bytes_failed;
	}

	*bufp += len;
	*count -= len;

	if((pod->chrdev_readpos += len) >= mlen)
		pod->chrdev_readpos = 0;
}

static void pod_check_chrdev_readbuf(struct usb_line6_pod *pod)
{
	if(pod->chrdev_readpos != 0)
		DEBUG_MESSAGES(dev_err(pod->line6.msgdev, "read buffer not empty\n"));
}

/*
	Read from PODxt Pro character device.
*/
static ssize_t pod_read(struct file *filp, char *buff, size_t count, loff_t *offp)
{
	struct usb_line6_pod *pod = (struct usb_line6_pod *)filp->private_data;
	unsigned char *bufp = (unsigned char *)buff;
	size_t retval = 0;
	int i;
	DECLARE_WAITQUEUE(wait, current);
	add_wait_queue(&pod->batch_wait, &wait);
	current->state = TASK_INTERRUPTIBLE;

	if(pod->chrdev_readpos > 0)
		pod_read_copy(pod, &bufp, &count);

	while(count > 0) {
		if(test_and_clear_bit(POD_CHANNEL_DIRTY, &pod->atomic_flags)) {
			pod_check_chrdev_readbuf(pod);
			pod->chrdev_readbuf[0] = LINE6_PROGRAM_CHANGE;
			pod->chrdev_readbuf[1] = pod->channel_num;
			pod->chrdev_msglen = 2;
			pod_read_copy(pod, &bufp, &count);
		}

		if(test_and_clear_bit(POD_SAVE_PRESSED, &pod->atomic_flags)) {
			static const char save_pressed[] = { 0xf5, 0x00, 0x01, 0x0c, 0x03, 0x24, 0x01, 0x05, 0xf7 };
			pod_check_chrdev_readbuf(pod);
			memcpy(pod->chrdev_readbuf, save_pressed, sizeof(save_pressed));
			pod->chrdev_msglen = sizeof(save_pressed);
			pod_read_copy(pod, &bufp, &count);
		}

		for(i = 0; (i < POD_CONTROL_SIZE) && (count > 0); ++i) {
			if(!test_and_clear_bit(i, pod->param_dirty))
				continue;

			pod_check_chrdev_readbuf(pod);
			pod->chrdev_readbuf[0] = LINE6_PARAM_CHANGE;
			pod->chrdev_readbuf[1] = i;
			pod->chrdev_readbuf[2] = pod->prog_data.control[i];
			pod->chrdev_msglen = 3;
			pod_read_copy(pod, &bufp, &count);
		}

		if((char *)bufp > buff)
			break;

		if(filp->f_flags & O_NONBLOCK) {
			retval = -EAGAIN;
			break;
		}

		if(signal_pending(current)) {
			retval = -ERESTARTSYS;
			break;
		}
		
		schedule();
	}

	current->state = TASK_RUNNING;
	remove_wait_queue(&pod->batch_wait, &wait);
	return (retval < 0) ? retval : ((char *)bufp - buff);
}

/*
	Write to PODxt Pro character device.
*/
static ssize_t pod_write(struct file *filp, const char *buff, size_t count, loff_t *offp)
{
	struct usb_line6_pod *pod = (struct usb_line6_pod *)filp->private_data;
	size_t size = pod->chrdev_writepos + count;
	size_t retval;
	unsigned char *bufp0, *bufp;
	int bytes_failed;

	/* check/wait for unfinished dump: */
	retval = line6_wait_dump(&pod->dumpreq, filp->f_flags & O_NONBLOCK);

	if(retval < 0)
		return retval;

	/* copy data to kernel space: */
	bufp0 = kmalloc(size, GFP_KERNEL);

	if(!bufp0) {
		dev_err(pod->line6.msgdev, "out of memory\n");
		return -ENOMEM;
	}

	bufp = bufp0;

	if(pod->chrdev_writepos > 0)
		memcpy(bufp, pod->chrdev_writebuf, pod->chrdev_writepos);

	bytes_failed = copy_from_user(bufp + pod->chrdev_writepos, buff, count);

	if(bytes_failed > 0) {
		DEBUG_MESSAGES(dev_err(pod->line6.msgdev, "couldn't copy data\n"));
		count -= bytes_failed;
	}

	pod->chrdev_writepos = 0;
	
	/* process requests: */
	while(size > 0) {
		unsigned char cmd = bufp[0];
		size_t mlen = pod_message_length(cmd);

		if(mlen < 0) {
			DEBUG_MESSAGES(dev_err(pod->line6.msgdev, "unknown command %02X\n", cmd));
			++bufp;
			continue;
		}

		if(mlen > size) {
			memcpy(pod->chrdev_writebuf, bufp, size);
			pod->chrdev_writepos = size;
			size = 0;
			break;
		}

		size -= mlen;

		switch(cmd) {
		case LINE6_PROGRAM_CHANGE:
			pod_send_channel(pod, bufp[1]);

			if(filp->f_flags & O_NONBLOCK)
				goto exit;  /* we don't need to report an error here since we already have written at least one byte */

			line6_wait_dump(&pod->dumpreq, 0);
			break;

		case LINE6_PARAM_CHANGE:
			pod_transmit_parameter(pod, bufp[1], bufp[2]);
			break;
		}

		bufp += mlen;
	}

 exit:
	kfree(bufp0);
	return count - size;
}

/*
	Open PODxt Pro character device.
*/
static int pod_open(struct inode *inode, struct file *filp)
{
	struct usb_line6_pod *pod;

	int num = MINOR(inode->i_rdev);

	if(num > LINE6_MAX_DEVICES)
		return -ENODEV;

	pod = line6_devices[num];

	if(pod == 0)
		return -ENODEV;

	/* check if device is busy: */
	switch(filp->f_flags & O_ACCMODE) {
	case O_RDONLY:
		if(test_and_set_bit(POD_BUSY_READ, &pod->atomic_flags))
			return -EBUSY;

		break;

	case O_WRONLY:
		if(test_and_set_bit(POD_BUSY_WRITE, &pod->atomic_flags))
			return -EBUSY;

		break;

	case O_RDWR:
		if(test_and_set_bit(POD_BUSY_READ, &pod->atomic_flags))
			return -EBUSY;

		if(test_and_set_bit(POD_BUSY_WRITE, &pod->atomic_flags)) {
			clear_bit(POD_BUSY_READ, &pod->atomic_flags);  /* not nice, can't we do an atomic operation on two bits? */
			return -EBUSY;
		}
	}

	filp->private_data = pod;
	return 0;
}

/*
	Release PODxt Pro character device.
*/
static int pod_release(struct inode *inode, struct file *filp)
{
	struct usb_line6_pod *pod = (struct usb_line6_pod *)filp->private_data;

	/* clear busy flags: */
	switch(filp->f_flags & O_ACCMODE) {
	case O_RDONLY:
		clear_bit(POD_BUSY_READ, &pod->atomic_flags);
		break;

	case O_WRONLY:
		clear_bit(POD_BUSY_WRITE, &pod->atomic_flags);
		break;

	case O_RDWR:
		clear_bit(POD_BUSY_READ, &pod->atomic_flags);
		clear_bit(POD_BUSY_WRITE, &pod->atomic_flags);
	}

	return 0;
}

/* character device file operations: */
static struct file_operations pod_fops = {
	read: pod_read,
	write: pod_write,
	open: pod_open,
	release: pod_release,
	owner: THIS_MODULE
};

#endif  /* USE_CHARACTER_DEVICE */

/*
	Resolve value to memory location.
*/
static void pod_resolve(const char *buf, short block0, short block1, unsigned char *location)
{
	int value = simple_strtoul(buf, NULL, 10);
	short block = (value < 0x40) ? block0 : block1;
	value &= 0x3f;
	location[0] = block >> 7;
	location[1] = value | (block & 0x7f);
}

/*
	Send command to store channel/effects setup/amp setup to PODxt Pro.
*/
static ssize_t pod_send_store_command(struct device *dev, const char *buf, size_t count, short block0, short block1)
{
	struct usb_interface *interface = to_usb_interface(dev);
	struct usb_line6_pod *pod = usb_get_intfdata(interface);

	int size = 3 + sizeof(pod->prog_data_buf);
	char *sysex = pod_alloc_sysex_buffer(pod, POD_SYSEX_STORE, size);
	if(!sysex) return 0;

	sysex[SYSEX_DATA_OFS] = 5;  /* see pod_dump() */
	pod_resolve(buf, block0, block1, sysex + SYSEX_DATA_OFS + 1);
	memcpy(sysex + SYSEX_DATA_OFS + 3, &pod->prog_data_buf, sizeof(pod->prog_data_buf));

	line6_send_sysex_message(&pod->line6, sysex, size);
	kfree(sysex);
	/* needs some delay here on AMD64 platform */
	return count;
}

/*
	Send command to retrieve channel/effects setup/amp setup to PODxt Pro.
*/
static ssize_t pod_send_retrieve_command(struct device *dev, const char *buf, size_t count, short block0, short block1)
{
	struct usb_interface *interface = to_usb_interface(dev);
	struct usb_line6_pod *pod = usb_get_intfdata(interface);

	int size = 4;
	char *sysex = pod_alloc_sysex_buffer(pod, POD_SYSEX_DUMPMEM, size);
	if(!sysex) return 0;

	pod_resolve(buf, block0, block1, sysex + SYSEX_DATA_OFS);
	sysex[SYSEX_DATA_OFS + 2] = 0;
	sysex[SYSEX_DATA_OFS + 3] = 0;
	line6_dump_started(&pod->dumpreq, POD_DUMP_MEMORY);

	if(line6_send_sysex_message(&pod->line6, sysex, size) < size)
		line6_dump_finished(&pod->dumpreq);

	kfree(sysex);
	/* needs some delay here on AMD64 platform */
	return count;
}

/*
	Generic get name function.
*/
static ssize_t get_name_generic(struct usb_line6_pod *pod, const char *str, char *buf)
{
	int length = 0;
	const char *p1;
	char *p2;
	char *last_non_space = buf;

	int retval = line6_wait_dump(&pod->dumpreq, 0);
	if(retval < 0) return retval;

	for(p1 = str, p2 = buf; *p1; ++p1, ++p2) {
		*p2 = *p1;
		if(*p2 != ' ') last_non_space = p2;
		if(++length == POD_NAME_LENGTH) break;
	}

	*(last_non_space + 1) = '\n';
	return last_non_space - buf + 2;
}

/*
	"read" request on "channel" special file.
*/
static ssize_t pod_get_channel(struct device *dev, DEVICE_ATTRIBUTE char *buf)
{
	struct usb_interface *interface = to_usb_interface(dev);
	struct usb_line6_pod *pod = usb_get_intfdata(interface);
	return sprintf(buf, "%d\n", pod->channel_num);
}

/*
	"write" request on "channel" special file.
*/
static ssize_t pod_set_channel(struct device *dev, DEVICE_ATTRIBUTE const char *buf, size_t count)
{
	struct usb_interface *interface = to_usb_interface(dev);
	struct usb_line6_pod *pod = usb_get_intfdata(interface);
	int value = simple_strtoul(buf, NULL, 10);
	pod_send_channel(pod, value);
	return count;
}

/*
	"read" request on "name" special file.
*/
static ssize_t pod_get_name(struct device *dev, DEVICE_ATTRIBUTE char *buf)
{
	struct usb_interface *interface = to_usb_interface(dev);
	struct usb_line6_pod *pod = usb_get_intfdata(interface);
	return get_name_generic(pod, pod->prog_data.header + POD_NAME_OFFSET, buf);
}

/*
	"read" request on "name" special file.
*/
static ssize_t pod_get_name_buf(struct device *dev, DEVICE_ATTRIBUTE char *buf)
{
	struct usb_interface *interface = to_usb_interface(dev);
	struct usb_line6_pod *pod = usb_get_intfdata(interface);
	return get_name_generic(pod, pod->prog_data_buf.header + POD_NAME_OFFSET, buf);
}

/*
	"read" request on "dump" special file.
*/
static ssize_t pod_get_dump(struct device *dev, DEVICE_ATTRIBUTE char *buf)
{
	struct usb_interface *interface = to_usb_interface(dev);
	struct usb_line6_pod *pod = usb_get_intfdata(interface);
	int retval = line6_wait_dump(&pod->dumpreq, 0);
	if(retval < 0) return retval;
	memcpy(buf, &pod->prog_data, sizeof(pod->prog_data));
	return sizeof(pod->prog_data);
}

/*
	"write" request on "dump" special file.
*/
static ssize_t pod_set_dump(struct device *dev, DEVICE_ATTRIBUTE const char *buf, size_t count)
{
	struct usb_interface *interface = to_usb_interface(dev);
	struct usb_line6_pod *pod = usb_get_intfdata(interface);

	if(count != sizeof(pod->prog_data)) {
		dev_err(pod->line6.msgdev,
						"data block must be exactly %d bytes\n",
						(int)sizeof(pod->prog_data));
		return -EINVAL;
	}

	pod_dump(pod, buf);
	return sizeof(pod->prog_data);
}

/*
	Request system parameter.
	@param tuner non-zero, if code refers to a tuner parameter
*/
static ssize_t pod_get_system_param(struct usb_line6_pod *pod, char *buf, int code, struct ValueWait *param, int tuner, int sign)
{
	char *sysex;
	int value;
	static const int size = 1;
	int retval = 0;
	DECLARE_WAITQUEUE(wait, current);

	if(((pod->prog_data.control[POD_tuner] & 0x40) == 0) && tuner)
		return -ENODEV;

	/* send value request to tuner: */
	param->value = POD_system_invalid;
	sysex = pod_alloc_sysex_buffer(pod, POD_SYSEX_SYSTEMREQ, size);
	if(!sysex) return 0;
	sysex[SYSEX_DATA_OFS] = code;
	line6_send_sysex_message(&pod->line6, sysex, size);
	kfree(sysex);

	/* wait for tuner to respond: */
	add_wait_queue(&param->wait, &wait);
	current->state = TASK_INTERRUPTIBLE;

	while(param->value == POD_system_invalid) {
		if(signal_pending(current)) {
			retval = -ERESTARTSYS;
			break;
		}
		else
			schedule();
	}

	current->state = TASK_RUNNING;
	remove_wait_queue(&param->wait, &wait);

	if(retval < 0)
		return retval;

	value = sign ? (int)(signed short)param->value : (int)(unsigned short)param->value;
	return sprintf(buf, "%d\n", value);
}

/*
	Send system parameter.
	@param tuner non-zero, if code refers to a tuner parameter
*/
static ssize_t pod_set_system_param(struct usb_line6_pod *pod, const char *buf, int count, int code, unsigned short mask, int tuner)
{
	char *sysex;
	static const int size = 5;
	unsigned short value;

	if(((pod->prog_data.control[POD_tuner] & 0x40) == 0) && tuner)
		return -EINVAL;

	/* send value to tuner: */
	sysex = pod_alloc_sysex_buffer(pod, POD_SYSEX_SYSTEM, size);
	if(!sysex) return 0;
	value = simple_strtoul(buf, NULL, 10) & mask;
	sysex[SYSEX_DATA_OFS] = code;
	sysex[SYSEX_DATA_OFS + 1] = (value >> 12) & 0x0f;
	sysex[SYSEX_DATA_OFS + 2] = (value >>  8) & 0x0f;
	sysex[SYSEX_DATA_OFS + 3] = (value >>  4) & 0x0f;
	sysex[SYSEX_DATA_OFS + 4] = (value      ) & 0x0f;
	line6_send_sysex_message(&pod->line6, sysex, size);
	kfree(sysex);
	return count;
}

/*
	"read" request on "dump_buf" special file.
*/
static ssize_t pod_get_dump_buf(struct device *dev, DEVICE_ATTRIBUTE char *buf)
{
	struct usb_interface *interface = to_usb_interface(dev);
	struct usb_line6_pod *pod = usb_get_intfdata(interface);
	int retval = line6_wait_dump(&pod->dumpreq, 0);
	if(retval < 0) return retval;
	memcpy(buf, &pod->prog_data_buf, sizeof(pod->prog_data_buf));
	return sizeof(pod->prog_data_buf);
}

/*
	"write" request on "dump_buf" special file.
*/
static ssize_t pod_set_dump_buf(struct device *dev, DEVICE_ATTRIBUTE const char *buf, size_t count)
{
	struct usb_interface *interface = to_usb_interface(dev);
	struct usb_line6_pod *pod = usb_get_intfdata(interface);

	if(count != sizeof(pod->prog_data)) {
		dev_err(pod->line6.msgdev,
						"data block must be exactly %d bytes\n",
						(int)sizeof(pod->prog_data));
		return -EINVAL;
	}
	
	memcpy(&pod->prog_data_buf, buf, sizeof(pod->prog_data));
	return sizeof(pod->prog_data);
}

/*
	"write" request on "finish" special file.
*/
static ssize_t pod_set_finish(struct device *dev, DEVICE_ATTRIBUTE const char *buf, size_t count)
{
	struct usb_interface *interface = to_usb_interface(dev);
	struct usb_line6_pod *pod = usb_get_intfdata(interface);
	int size = 0;
	char *sysex = pod_alloc_sysex_buffer(pod, POD_SYSEX_FINISH, size);
	if(!sysex) return 0;
	line6_send_sysex_message(&pod->line6, sysex, size);
	kfree(sysex);
	return count;
}

/*
	"write" request on "store_channel" special file.
*/
static ssize_t pod_set_store_channel(struct device *dev, DEVICE_ATTRIBUTE const char *buf, size_t count)
{
	return pod_send_store_command(dev, buf, count, 0x0000, 0x00c0);
}

/*
	"write" request on "store_effects_setup" special file.
*/
static ssize_t pod_set_store_effects_setup(struct device *dev, DEVICE_ATTRIBUTE const char *buf, size_t count)
{
	return pod_send_store_command(dev, buf, count, 0x0080, 0x0080);
}

/*
	"write" request on "store_amp_setup" special file.
*/
static ssize_t pod_set_store_amp_setup(struct device *dev, DEVICE_ATTRIBUTE const char *buf, size_t count)
{
	return pod_send_store_command(dev, buf, count, 0x0040, 0x0100);
}

/*
	"write" request on "retrieve_channel" special file.
*/
static ssize_t pod_set_retrieve_channel(struct device *dev, DEVICE_ATTRIBUTE const char *buf, size_t count)
{
	return pod_send_retrieve_command(dev, buf, count, 0x0000, 0x00c0);
}

/*
	"write" request on "retrieve_effects_setup" special file.
*/
static ssize_t pod_set_retrieve_effects_setup(struct device *dev, DEVICE_ATTRIBUTE const char *buf, size_t count)
{
	return pod_send_retrieve_command(dev, buf, count, 0x0080, 0x0080);
}

/*
	"write" request on "retrieve_amp_setup" special file.
*/
static ssize_t pod_set_retrieve_amp_setup(struct device *dev, DEVICE_ATTRIBUTE const char *buf, size_t count)
{
	return pod_send_retrieve_command(dev, buf, count, 0x0040, 0x0100);
}

/*
	"read" request on "dirty" special file.
*/
static ssize_t pod_get_dirty(struct device *dev, DEVICE_ATTRIBUTE char *buf)
{
	struct usb_interface *interface = to_usb_interface(dev);
	struct usb_line6_pod *pod = usb_get_intfdata(interface);
	buf[0] = pod->dirty ? '1' : '0';
	buf[1] = '\n';
	return 2;
}

/*
	"read" request on "midi_postprocess" special file.
*/
static ssize_t pod_get_midi_postprocess(struct device *dev, DEVICE_ATTRIBUTE char *buf)
{
	struct usb_interface *interface = to_usb_interface(dev);
	struct usb_line6_pod *pod = usb_get_intfdata(interface);
	return sprintf(buf, "%d\n", pod->midi_postprocess);
}

/*
	"write" request on "midi_postprocess" special file.
*/
static ssize_t pod_set_midi_postprocess(struct device *dev, DEVICE_ATTRIBUTE const char *buf, size_t count)
{
	struct usb_interface *interface = to_usb_interface(dev);
	struct usb_line6_pod *pod = usb_get_intfdata(interface);
	int value = simple_strtoul(buf, NULL, 10);
	pod->midi_postprocess = value ? 1 : 0;
	return count;
}

/*
	"read" request on "serial_number" special file.
*/
static ssize_t pod_get_serial_number(struct device *dev, DEVICE_ATTRIBUTE char *buf)
{
	struct usb_interface *interface = to_usb_interface(dev);
	struct usb_line6_pod *pod = usb_get_intfdata(interface);
	sprintf(buf, "%d\n", pod->serial_number);
	return strlen(buf);
}

/*
	"read" request on "firmware_version" special file.
*/
static ssize_t pod_get_firmware_version(struct device *dev, DEVICE_ATTRIBUTE char *buf)
{
	struct usb_interface *interface = to_usb_interface(dev);
	struct usb_line6_pod *pod = usb_get_intfdata(interface);
	sprintf(buf, "%d.%02d\n", pod->firmware_version / 100, pod->firmware_version % 100);
	return strlen(buf);
}

/*
	"read" request on "device_id" special file.
*/
static ssize_t pod_get_device_id(struct device *dev, DEVICE_ATTRIBUTE char *buf)
{
	struct usb_interface *interface = to_usb_interface(dev);
	struct usb_line6_pod *pod = usb_get_intfdata(interface);
	sprintf(buf, "%d\n", pod->device_id);
	return strlen(buf);
}

#define POD_GET_SYSTEM_PARAM(code, tuner, sign) \
static ssize_t pod_get_ ## code(struct device *dev, DEVICE_ATTRIBUTE char *buf) \
{ \
	struct usb_interface *interface = to_usb_interface(dev); \
	struct usb_line6_pod *pod = usb_get_intfdata(interface); \
	return pod_get_system_param(pod, buf, POD_ ## code, &pod->code, tuner, sign); \
}

#define POD_GET_SET_SYSTEM_PARAM(code, mask, tuner, sign) \
POD_GET_SYSTEM_PARAM(code, tuner, sign) \
static ssize_t pod_set_ ## code(struct device *dev, DEVICE_ATTRIBUTE const char *buf, size_t count) \
{ \
	struct usb_interface *interface = to_usb_interface(dev); \
	struct usb_line6_pod *pod = usb_get_intfdata(interface); \
	return pod_set_system_param(pod, buf, count, POD_ ## code, mask, tuner); \
}

POD_GET_SET_SYSTEM_PARAM(monitor_level, 0xffff, 0, 0);
POD_GET_SET_SYSTEM_PARAM(routing, 0x0003, 0, 0);
POD_GET_SET_SYSTEM_PARAM(tuner_mute, 0x0001, 1, 0);
POD_GET_SET_SYSTEM_PARAM(tuner_freq, 0xffff, 1, 0);
POD_GET_SYSTEM_PARAM(tuner_note, 1, 1);
POD_GET_SYSTEM_PARAM(tuner_pitch, 1, 1);

#undef GET_SET_SYSTEM_PARAM
#undef GET_SYSTEM_PARAM

/* POD special files: */
static DEVICE_ATTR(channel, S_IWUGO | S_IRUGO, pod_get_channel, pod_set_channel);
static DEVICE_ATTR(name, S_IRUGO, pod_get_name, line6_nop_write);
static DEVICE_ATTR(name_buf, S_IRUGO, pod_get_name_buf, line6_nop_write);
static DEVICE_ATTR(dump, S_IWUGO | S_IRUGO, pod_get_dump, pod_set_dump);
static DEVICE_ATTR(dump_buf, S_IWUGO | S_IRUGO, pod_get_dump_buf, pod_set_dump_buf);
static DEVICE_ATTR(store_channel, S_IWUGO, line6_nop_read, pod_set_store_channel);
static DEVICE_ATTR(store_effects_setup, S_IWUGO, line6_nop_read, pod_set_store_effects_setup);
static DEVICE_ATTR(store_amp_setup, S_IWUGO, line6_nop_read, pod_set_store_amp_setup);
static DEVICE_ATTR(finish, S_IWUGO, line6_nop_read, pod_set_finish);
static DEVICE_ATTR(retrieve_channel, S_IWUGO, line6_nop_read, pod_set_retrieve_channel);
static DEVICE_ATTR(retrieve_effects_setup, S_IWUGO, line6_nop_read, pod_set_retrieve_effects_setup);
static DEVICE_ATTR(retrieve_amp_setup, S_IWUGO, line6_nop_read, pod_set_retrieve_amp_setup);
static DEVICE_ATTR(dirty, S_IRUGO, pod_get_dirty, line6_nop_write);
static DEVICE_ATTR(monitor_level, S_IWUGO | S_IRUGO, pod_get_monitor_level, pod_set_monitor_level);
static DEVICE_ATTR(routing, S_IWUGO | S_IRUGO, pod_get_routing, pod_set_routing);
static DEVICE_ATTR(tuner_mute, S_IWUGO | S_IRUGO, pod_get_tuner_mute, pod_set_tuner_mute);
static DEVICE_ATTR(tuner_freq, S_IWUGO | S_IRUGO, pod_get_tuner_freq, pod_set_tuner_freq);
static DEVICE_ATTR(tuner_note, S_IRUGO, pod_get_tuner_note, line6_nop_write);
static DEVICE_ATTR(tuner_pitch, S_IRUGO, pod_get_tuner_pitch, line6_nop_write);
static DEVICE_ATTR(serial_number, S_IRUGO, pod_get_serial_number, line6_nop_write);
static DEVICE_ATTR(firmware_version, S_IRUGO, pod_get_firmware_version, line6_nop_write);
static DEVICE_ATTR(device_id, S_IRUGO, pod_get_device_id, line6_nop_write);
static DEVICE_ATTR(midi_postprocess, S_IWUGO | S_IRUGO, pod_get_midi_postprocess, pod_set_midi_postprocess);
#if CREATE_RAW_FILE
static DEVICE_ATTR(raw, S_IWUGO, line6_nop_read, line6_set_raw);
#endif

/*
	POD destructor.
*/
static void pod_destruct(struct usb_interface *interface)
{
	struct usb_line6_pod *pod = usb_get_intfdata(interface);

	if(pod == NULL)
		return;

	line6_cleanup_audio(&pod->line6);

#if USE_CHARACTER_DEVICE
	/* unregister character device: */
	if(pod->chrdev >= 0) unregister_chrdev(POD_CHRDEV_MAJOR, POD_CHRDEV_NAME);
#endif

	/* free buffer memory first: */
	line6_dumpreq_destruct(&pod->dumpreq);
	if(pod->buffer_versionreq) kfree(pod->buffer_versionreq);
}

/**
	 Init POD device.
*/
int pod_init(struct usb_interface *interface, struct usb_line6_pod *pod)
{
	int err;

	pod->channel_num = 255;
#if USE_CHARACTER_DEVICE
	pod->chrdev = -1;
#endif

	init_waitqueue_head(&pod->monitor_level.wait);
	init_waitqueue_head(&pod->routing.wait);
	init_waitqueue_head(&pod->tuner_mute.wait);
	init_waitqueue_head(&pod->tuner_freq.wait);
	init_waitqueue_head(&pod->tuner_note.wait);
	init_waitqueue_head(&pod->tuner_pitch.wait);

	/* initialize batch data system: */
	init_waitqueue_head(&pod->batch_wait);
	memset(pod->param_dirty, 0xff, sizeof(pod->param_dirty));

	/* initialize USB buffers: */
	err = line6_dumpreq_init(&pod->dumpreq, pod_request_channel, sizeof(pod_request_channel));

	if(err < 0) {
		dev_err(&interface->dev, "Out of memory\n");
		pod_destruct(interface);
		return -ENOMEM;
	}

	pod->buffer_versionreq = kmalloc(sizeof(pod_request_version), GFP_KERNEL);

	if(pod->buffer_versionreq == NULL) {
		dev_err(&interface->dev, "Out of memory\n");
		pod_destruct(interface);
		return -ENOMEM;
	}

	memcpy(pod->buffer_versionreq, pod_request_version, sizeof(pod_request_version));

#if USE_CHARACTER_DEVICE
	if((pod->chrdev = register_chrdev(POD_CHRDEV_MAJOR, POD_CHRDEV_NAME, &pod_fops)) < 0) {
		pod_destruct(interface);
		return -EINVAL;
	}
#endif

	/* create sysfs entries: */
	device_create_file(&interface->dev, &dev_attr_channel);
	device_create_file(&interface->dev, &dev_attr_name);
	device_create_file(&interface->dev, &dev_attr_name_buf);
	device_create_file(&interface->dev, &dev_attr_dump);
	device_create_file(&interface->dev, &dev_attr_dump_buf);
	device_create_file(&interface->dev, &dev_attr_store_channel);
	device_create_file(&interface->dev, &dev_attr_store_effects_setup);
	device_create_file(&interface->dev, &dev_attr_store_amp_setup);
	device_create_file(&interface->dev, &dev_attr_finish);
	device_create_file(&interface->dev, &dev_attr_retrieve_channel);
	device_create_file(&interface->dev, &dev_attr_retrieve_effects_setup);
	device_create_file(&interface->dev, &dev_attr_retrieve_amp_setup);
	device_create_file(&interface->dev, &dev_attr_dirty);
	device_create_file(&interface->dev, &dev_attr_monitor_level);
	device_create_file(&interface->dev, &dev_attr_routing);
	device_create_file(&interface->dev, &dev_attr_tuner_mute);
	device_create_file(&interface->dev, &dev_attr_tuner_freq);
	device_create_file(&interface->dev, &dev_attr_tuner_note);
	device_create_file(&interface->dev, &dev_attr_tuner_pitch);
	device_create_file(&interface->dev, &dev_attr_serial_number);
	device_create_file(&interface->dev, &dev_attr_firmware_version);
	device_create_file(&interface->dev, &dev_attr_device_id);
	device_create_file(&interface->dev, &dev_attr_midi_postprocess);

#if CREATE_RAW_FILE
	device_create_file(&interface->dev, &dev_attr_raw);
#endif

	/* initialize audio system: */
	if((err = line6_init_audio(&pod->line6)) < 0) {
		pod_destruct(interface);
		return err;
	}

	/* initialize PCM subsystem: */
	if((err = pod_init_pcm(pod)) < 0) {
		pod_destruct(interface);
		return err;
	}

	/* register audio system: */
	if((err = line6_register_audio(&pod->line6)) < 0) {
		pod_destruct(interface);
		return err;
	}

	/* query some data: */
	line6_startup_delayed(&pod->dumpreq, 3, pod_startup_timeout, pod);
	pod_read_serial_number(pod);
	return 0;
}

/*
	POD device disconnected.
*/
void pod_disconnect(struct usb_interface *interface)
{
	struct usb_line6_pod *pod = usb_get_intfdata(interface);

	unlink_wait_clear_audio_out_urbs(pod->chip);
	unlink_wait_clear_audio_in_urbs(pod->chip);

	/* remove sysfs entries: */
	if(pod->versionreq_ok)
		pod_remove_files(pod->firmware_version, pod->line6.devbit, &interface->dev);

	device_remove_file(&interface->dev, &dev_attr_channel);
	device_remove_file(&interface->dev, &dev_attr_name);
	device_remove_file(&interface->dev, &dev_attr_name_buf);
	device_remove_file(&interface->dev, &dev_attr_dump);
	device_remove_file(&interface->dev, &dev_attr_dump_buf);
	device_remove_file(&interface->dev, &dev_attr_store_channel);
	device_remove_file(&interface->dev, &dev_attr_store_effects_setup);
	device_remove_file(&interface->dev, &dev_attr_store_amp_setup);
	device_remove_file(&interface->dev, &dev_attr_finish);
	device_remove_file(&interface->dev, &dev_attr_retrieve_channel);
	device_remove_file(&interface->dev, &dev_attr_retrieve_effects_setup);
	device_remove_file(&interface->dev, &dev_attr_retrieve_amp_setup);
	device_remove_file(&interface->dev, &dev_attr_dirty);
	device_remove_file(&interface->dev, &dev_attr_monitor_level);
	device_remove_file(&interface->dev, &dev_attr_routing);
	device_remove_file(&interface->dev, &dev_attr_tuner_mute);
	device_remove_file(&interface->dev, &dev_attr_tuner_freq);
	device_remove_file(&interface->dev, &dev_attr_tuner_note);
	device_remove_file(&interface->dev, &dev_attr_tuner_pitch);
	device_remove_file(&interface->dev, &dev_attr_serial_number);
	device_remove_file(&interface->dev, &dev_attr_firmware_version);
	device_remove_file(&interface->dev, &dev_attr_device_id);
	device_remove_file(&interface->dev, &dev_attr_midi_postprocess);
#if CREATE_RAW_FILE
	device_remove_file(&interface->dev, &dev_attr_raw);
#endif

	pod_destruct(interface);
}
